<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs"
    xmlns:xnl="urn:oasis:names:tc:ciq:xnl:4"
    xmlns:xal="urn:oasis:names:tc:ciq:xal:4"
   version="2.0">
    
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Author: David RR Webber, December, 2010 -->
    <!--
        Purpose  : Read in Google VIP project election XML and output OASIS EML standard XML instances. 
        Original : 12/11/2010
        Initials : drrw
        CAM ver    : 1.1 
        Description : Part of a set of tools to provide open source tools for election management using OASIS EML.
        Updated  - Inits - Change
        12/11/10   drrw    Initial release
        
    -->
    <xsl:param name="ownerFileName">Virginia</xsl:param>
    <xsl:param name="electionYear">2010</xsl:param>
    <xsl:param name="electionName">Virginia Election 2010</xsl:param>
    
    <xsl:output indent="yes"/>
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="EMLfiles" method="xml" indent="yes" omit-xml-declaration="yes"/>
    
    <xsl:variable name="version">1.00</xsl:variable>
    <xsl:variable name="nbsp" ><xsl:text disable-output-escaping="yes">&#160;&#160;</xsl:text></xsl:variable>
   
   <xsl:variable name="VIPfeed" select="/"/>
    
    <xsl:template match="/">
        <!-- Compute actual output path to write to --> 
      
        <xsl:variable name="writeResultPath110" select="concat($ownerFileName,'-EML110','.xml')"/>
        <xsl:variable name="eventIdentifier" select="distinct-values(vip_object/contest/election_id)"></xsl:variable>
        
        <!-- Output Election information -->
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath110}">
            
            <EML Id="110" SchemaVersion="6.0">
                <xsl:comment>
                    <xsl:value-of select="$nbsp"/>
                    Total Contests: <xsl:value-of select="count(vip_object/contest)"/>
                    Total Districts: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    <xsl:value-of select="$nbsp"/>
                    <xsl:value-of select="$nbsp"/>
                     -----------------------------------------
                </xsl:comment>
                <TransactionId>1</TransactionId>
                <SequenceNumber>1</SequenceNumber>
                <NumberInSequence>1</NumberInSequence>
                <SequencedElementName/>
                <MessageLanguage>en-US</MessageLanguage>
                <RequestedResponseLanguage>en-US</RequestedResponseLanguage>
                <IssueDate><xsl:value-of select="current-date()"/></IssueDate>
                <ElectionEvent>
                    <EventIdentifier IdNumber="{$eventIdentifier}" DisplayOrder="1">
                        <EventName><xsl:value-of select="$electionYear"/> Election</EventName>
                        <EventQualifier IdNumber="{$electionYear}">General Election</EventQualifier>
                    </EventIdentifier>
                    <DocumentIdentifier Href="http://www.va.gov/SBE">Election Announcement</DocumentIdentifier>
                    <Languages>
                        <Language>en-US</Language>
                        <DefaultLanguage>en-US</DefaultLanguage>
                    </Languages>
                    <Election>
                        <ElectionIdentifier ShortCode="{$electionYear}" IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <ElectionName><xsl:value-of select="$electionName"/></ElectionName>
                            <ElectionCategory>General</ElectionCategory>
                        </ElectionIdentifier>
                        <DateRange Type="Token">
                            <Start><xsl:value-of select="current-date()"/></Start>
                            <End><xsl:value-of select="current-date()"/></End>
                            <SingleDate><xsl:value-of select="current-date()"/></SingleDate>
                        </DateRange>
                        <OfficialStatusDetail>
                            <OfficialStatus>Approved</OfficialStatus>
                            <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                        </OfficialStatusDetail>
                        <!-- Generate Context Details -->
                        <xsl:apply-templates select="vip_object/contest"/>
                    </Election>
                    </ElectionEvent>
            </EML>
           
        </xsl:result-document>
        
        <xsl:variable name="writeResultPath230" select="concat($ownerFileName,'-EML230','.xml')"/>
        <!-- Output Candidate Information -->
        
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath230}">
            
            <EML Id="230" SchemaVersion="6.0">
                <xsl:comment>
                    <xsl:value-of select="$nbsp"/>
                    Total Candidates: <xsl:value-of select="count(vip_object/candidate)"/>
                    Total Candidate Id: <xsl:value-of select="count(distinct-values(vip_object/candidate/@id))"/>
                    Candidates Addresses: <xsl:value-of select="count(vip_object/candidate/filed_mailing_address[line1 &gt; ' '])"></xsl:value-of>
                    Total Districts: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    <xsl:value-of select="$nbsp"/>
                    <xsl:value-of select="$nbsp"/>
                    -----------------------------------------
                </xsl:comment>
                <TransactionId>Token</TransactionId>
                <SequenceNumber>999999</SequenceNumber>
                <NumberInSequence>767113</NumberInSequence>
                <SequencedElementName>WordToken</SequencedElementName>
                <MessageLanguage>en-US</MessageLanguage>
                <IssueDate>2010-12-16T11:46:53.0-05:00</IssueDate>
                <CandidateList>
                    <EventIdentifier IdNumber="xs:NMTOKEN" DisplayOrder="1">
                        <EventName>Token</EventName>
                        <EventQualifier IdNumber="WordToken">Token</EventQualifier>
                    </EventIdentifier>
                    <EventDate>
                        <Start>2010-12-16T11:46:53.0-05:00</Start>
                        <End>2010-12-16T11:46:53.0-05:00</End>
                    </EventDate>
                    <ListDate>2010-12-16T11:46:53.0-05:00</ListDate>
                    <OfficialStatusDetail>
                        <OfficialStatus>type=OfficialStatusType</OfficialStatus>
                        <StatusDate>2010-12-16T11:46:53.0-05:00</StatusDate>
                    </OfficialStatusDetail>
                    <Election>
                        <ElectionIdentifier ShortCode="xs:NMTOKEN" IdNumber="xs:NMTOKEN" DisplayOrder="1">
                            <ElectionName>Token</ElectionName>
                            <ElectionGroup Id="Token">Token</ElectionGroup>
                            <ElectionCategory>Token</ElectionCategory>
                        </ElectionIdentifier>
                        <Contest>
                            <ContestIdentifier ShortCode="ALL" IdNumber="0000" DisplayOrder="1">
                                <ContestName>ALL CONTESTS</ContestName>
                            </ContestIdentifier>
                            <Description DisplayOrder="1">
                                <Message Lang="en-US">This is the consolidated list of all candidates for the election.</Message>
                            </Description>
                    <xsl:apply-templates select="vip_object/candidate"/>
                        </Contest>
                        </Election>
                    </CandidateList>
            </EML>
        </xsl:result-document>
        
        <xsl:variable name="writeResultPath505" select="concat($ownerFileName,'-EML505','.xml')"/>
        <!-- Output Candidate Information -->
        
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath505}">
          
            <EML xmlns:camed="http://jcam.org.uk/editor" xmlns:xnl="urn:oasis:names:tc:ciq:xnl:4"
     xmlns:as="http://www.oasis-open.org/committees/cam"
     xmlns:xal="urn:oasis:names:tc:ciq:xal:4"
     xmlns:xsd="http://www.w3.org/2001/XMLSchema"
     Id="505"
     SchemaVersion="7.0">
                <xsl:comment>
                    <xsl:value-of select="$nbsp"/>
                    Total Localities: <xsl:value-of select="count(vip_object/locality)"/>
                    Total Districts: <xsl:value-of select="count(vip_object/electoral_district)"/>
                    Total Districts in Contests: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Contests: <xsl:value-of select="count(vip_object/contest)"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/ballot))"/>
                    Total Ballots in Contests: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    Total Candidates: <xsl:value-of select="count(vip_object/candidate)"/>
                    Total Candidate Id: <xsl:value-of select="count(distinct-values(vip_object/candidate/@id))"/>
                    Candidates Addresses: <xsl:value-of select="count(vip_object/candidate/filed_mailing_address[line1 &gt; ' '])"></xsl:value-of>
                    Total Ballot measures: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    Total Polling Locations: <xsl:value-of select="count(distinct-values(vip_object/polling_location))"/>
                    Total Locality Boundaries: <xsl:value-of select="count(distinct-values(vip_object/locality_boundary))"/>
                    <xsl:value-of select="$nbsp"/>
                    <xsl:value-of select="$nbsp"/>
                    -----------------------------------------
                </xsl:comment>
   <EMLHeader>
      <TransactionId>505</TransactionId>
      <SequenceNumber>00001</SequenceNumber>
      <NumberInSequence>01</NumberInSequence>
      <MessageLanguage>en-US</MessageLanguage>
      <ManagingAuthority>
          <AuthorityIdentifier><xsl:value-of select="vip_object/state/name"/></AuthorityIdentifier>
          <AuthorityName Type="StateElectionBoard" id="{vip_object/source/vip_id}" usage="public" status="draft" LanguageCode="en-US">
            <xnl:NameElement ElementType="NameOnly">Virginia State Board of Elections</xnl:NameElement>
         </AuthorityName>
         <Description><xsl:value-of select="vip_object/source/description"/></Description>
          <OrganizationURL><xsl:value-of select="vip_object/source/organization_url"/></OrganizationURL>
         <AuthorityAddress type="Business">
            <xal:PostalAddress>
               <xal:Thoroughfare>100 Main Street</xal:Thoroughfare>
               <xal:OtherDetail>Suite #501</xal:OtherDetail>
                <xal:PostalDeliveryPoint type="other">Election Board Offices</xal:PostalDeliveryPoint>
               <xal:Locality type="City">Richmond</xal:Locality>
               <xal:AdministrativeArea type="State">VA</xal:AdministrativeArea>
               <xal:PostCode type="ZIP">20123</xal:PostCode>
               <xal:Country code="US">United States</xal:Country>
            </xal:PostalAddress>
         </AuthorityAddress>
      </ManagingAuthority>
       <IssueDate><xsl:value-of select="vip_object/source/datetime"/></IssueDate>
      <OfficialStatusDetail>
         <OfficialStatus>draft</OfficialStatus>
         <StatusDate><xsl:value-of select="vip_object/source/datetime"/></StatusDate>
      </OfficialStatusDetail>
   </EMLHeader>        
                
   <ElectionDetails>
       <Election id="{vip_object/election/@id}">
         <ElectionIdentifier ShortCode="2010" IdNumber="VA2010" DisplayOrder="1">
            <ElectionName>Virginia General Election</ElectionName>
            <ElectionGroup Id="Token">Token</ElectionGroup>
             <ElectionCategory><xsl:value-of select="vip_object/election/election_type"/></ElectionCategory>
         </ElectionIdentifier>
         <ElectionDate Type="Token">
            <Start>2010-10-05T08:00:00.0-05:00</Start>
             <End>2010-10-05T20:00:00.0-05:00</End>
             <SingleDate><xsl:value-of select="vip_object/election/date"/></SingleDate>
         </ElectionDate>
       <ElectionScope><xsl:value-of select="if(vip_object/election/statewide='YES') then 'Statewide' else 'Partial'"/></ElectionScope>
       <ElectionType><xsl:value-of select="vip_object/election/election_type"/></ElectionType>
       <ElectionJurisdiction><xsl:value-of select="vip_object/state/name"/></ElectionJurisdiction>
         <Localities>           
             <!-- Output Locality information -->
                <xsl:apply-templates select="vip_object/locality" mode="EML505"/>            
            <Precincts>
               <xsl:apply-templates select="vip_object/precinct" mode="EML505"/> 
            </Precincts>
            <PrecinctSplits>
               <xsl:apply-templates select="vip_object/precinct_split" mode="EML505"/> 
            </PrecinctSplits>
         </Localities>
         <Districts>
             <xsl:apply-templates select="vip_object/electoral_district" mode="EML505"/>                 
         </Districts>
         <Contests>
            <xsl:apply-templates select="vip_object/contest" mode="EML505"/>   
         </Contests>
         <Ballots>
            <xsl:apply-templates select="vip_object/ballot" mode="EML505a"/>
         </Ballots>
         <Candidates id="{vip_object/election/@id}">
            <xsl:apply-templates select="vip_object/candidate" mode="EML505"/>
         </Candidates>
          
         <Propositions>
            <Proposition ReferendumOptionIdentifier="Token" ProposalIdentifier="Token" Lang="en-US">
               <ProposalText>string</ProposalText>
               <SelectionText>string</SelectionText>
               <StatementType>for</StatementType>
               <PassThreshold>113876</PassThreshold>
               <AbstainEffect>string</AbstainEffect>
            </Proposition>
         </Propositions>
          
         <PollingLocations>
            <xsl:apply-templates select="vip_object/polling_location" mode="EML505"/>
         </PollingLocations>
          
         <LocalityBoundaries>
            <xsl:apply-templates select="vip_object/street_segment" mode="EML505"/>
         </LocalityBoundaries>
      </Election>
   </ElectionDetails>
           </EML> 
        </xsl:result-document>
        
    </xsl:template>
    
    <xsl:template match="contest">
        <Contest DisplayOrder="{position()}">
            <ContestIdentifier ShortCode="{./@id}" IdNumber="{./@id}" DisplayOrder="1">
                <ContestName><xsl:value-of select="./office"/></ContestName>
            </ContestIdentifier>
            <District>
                <ReportingUnitIdentifier IdNumber="{./electoral_district_id}" DisplayOrder="1"><xsl:value-of select="./electoral_district_id"/></ReportingUnitIdentifier>
                <Split>true</Split>
            </District>
            <Area IdNumber="1" DisplayOrder="1">County</Area>
            <Position IdNumber="{./ballot_placement}" DisplayOrder="{./ballot_placement}"/>
            <VotingMethod>FFP</VotingMethod>
            <xsl:if test="./partisan='Yes'"><VotingMethod>Partisan</VotingMethod></xsl:if>
            <ContestType>straight party</ContestType>
            <MaxVotes>999</MaxVotes>
            <MaxWriteIn>999</MaxWriteIn>
            <NumberOfPositions>999</NumberOfPositions>
            <Messages>
                <Message Format="Text" Type="Voter" Lang="en-US">Please complete UOCAVA attestation statement.</Message>
            </Messages>
        </Contest>
        <!-- xsl:copy-of select="."/ -->
        <!-- contest id="500000">
            <election_id>14</election_id>
            <type>General</type>
            <electoral_district_id>501698</electoral_district_id>
            <partisan>No</partisan>
            <primary_party>&#x20;</primary_party>
            <office>Member School Board - Bayside</office>
            <ballot_id>500245</ballot_id>
            <ballot_placement>28</ballot_placement>
        </contest -->
    </xsl:template>
    
    <xsl:template match="candidate">
        <Affliation>
          <AffliationIdentifier ShortCode="{party}">
              <RegisteredName><xsl:value-of select="party"/></RegisteredName>
              <Type>Party</Type>
          </AffliationIdentifier>
          <Candidate DisplayOrder="{position()}" Independent="{if(party='Independent') then 'yes' else 'no'}">
            <CandidateIdentifier IdNumber="{@id}" DisplayOrder="1">
                <CandidateName><xsl:value-of select="name"/></CandidateName>
                <KnownAs/>
                <CandidateGender/>
                <ShortCode/>
            </CandidateIdentifier>
            <StatusDetails>
                <StatusItem setting="confirmed" date="{current-date()}"/>
            </StatusDetails>
            <DateOfBirth/>
            <Age/>
            <Gender/>
            <QualifyingAddress Type="Residence" ValidFrom="{current-date()}" LanguageCode="en-US">
                <xal:Locality Type="PostTown">
                    <xal:NameElement NameType="Name"><xsl:value-of select="filed_mailing_address/city"/></xal:NameElement>
                </xal:Locality>
                <xal:Thoroughfare>
                    <xal:NameElement><xsl:value-of select="filed_mailing_address/line1"/></xal:NameElement>
                    <xal:Number><xsl:value-of select="tokenize(filed_mailing_address/line1,' ')[1]"/></xal:Number>
                </xal:Thoroughfare>
                <xal:PostCode>
                    <xal:Identifier Type="ZIP"><xsl:value-of select="filed_mailing_address/zip"/></xal:Identifier>
                </xal:PostCode>
                <xal:AdminstrativeArea Type="State" ValidFrom="{current-date()}" LanguageCode="en-US">
                    <xal:NameElement><xsl:value-of select="filed_mailing_address/state"/></xal:NameElement>
                </xal:AdminstrativeArea>
            </QualifyingAddress>
            <Contact DisplayOrder="1">
                <Email Preferred="yes"><xsl:value-of select="email"/></Email>
                <Telephone/>
                <Fax/>               
            </Contact>
            <Profession/>
        </Candidate>
       </Affliation> 
        
        <!-- xsl:copy-of select="."/ -->
        <!-- 
            <candidate id="500430">
            <name> Richard N. Early, Jr.</name>
            <party>Independent</party>
            <filed_mailing_address>
            <line1>237 Madison Rd</line1>
            <city>Stanardsville</city>
            <state>VA</state>
            <zip>229732933</zip>
            </filed_mailing_address>
            </candidate>        -->
    </xsl:template>
    
    <xsl:template match="locality" mode="EML505">
        <Locality id="{./@id}" xsl:exclude-result-prefixes="xal xnl">
            <LocalityName divisionType="locality"><xsl:value-of select="./name"/></LocalityName>
            <LocalityType><xsl:value-of select="./type"/></LocalityType>
           <LocalityID>0-<xsl:value-of select="./@id"/></LocalityID>
        </Locality>
    </xsl:template>
   
   <xsl:template match="precinct" mode="EML505"><!-- this matches precinct details -->
      <Precinct id="{@id}"  xsl:exclude-result-prefixes="xal xnl">
         <PrecinctName><xsl:value-of select="./name"/></PrecinctName>
         <LocalityID><xsl:value-of select="./locality_id"/></LocalityID>
         <ElectoralDistrictID/>
         <PollingLocationID><xsl:value-of select="./polling_location_id"/></PollingLocationID>
      </Precinct>
   </xsl:template>
   
   <xsl:template match="precinct_split" mode="EML505"><!-- this matches precinct details -->
    <PrecinctSplit id="{@id}"  xsl:exclude-result-prefixes="xal xnl">
       <SplitPrecinctName><xsl:value-of select="./name"/></SplitPrecinctName>
       <PrecinctID><xsl:value-of select="./precinct_id"/></PrecinctID>
       <xsl:apply-templates select="electoral_district_id" mode="EML505"/>
   </PrecinctSplit>
   </xsl:template>
   
   <xsl:template match="electoral_district_id" mode="EML505"><!-- this matches precinct details -->
   <ElectoralDistrictID  xsl:exclude-result-prefixes="xal xnl"><xsl:value-of select="."/></ElectoralDistrictID>
   </xsl:template>
   
    <xsl:template match="electoral_district" mode="EML505">
        <ElectoralDistrict id="{./@id}" xsl:exclude-result-prefixes="xal xnl">
            <DistrictName><xsl:value-of select="./name"/></DistrictName>
            <DistrictType><xsl:value-of select="./type"/></DistrictType>
        </ElectoralDistrict>
    </xsl:template>
   
   <xsl:template match="contest" mode="EML505">
      <Contest id="{position()}"  xsl:exclude-result-prefixes="xal xnl">
      <ContestType DisplayOrder="{./ballot_placement}" Completed="no"  xsl:exclude-result-prefixes="xal xnl">
         <ContestIdentifier ShortCode="{./ballot_id}" IdNumber="{./@id}" DisplayOrder="{./ballot_placement}">
            <ContestName><xsl:value-of select="./office"/></ContestName>
         </ContestIdentifier>
         <ContestScope><xsl:value-of select="if(string-length(./primary_party) &gt; 3) then ./primary_party else 'Statewide'"/></ContestScope>
         <ElectionType><xsl:value-of select="./type"/></ElectionType>
         <ReportingUnitIdentifier IdNumber="{./electoral_district_id}" DisplayOrder="1">District</ReportingUnitIdentifier>
         <Qualified Reason="approved">yes</Qualified>
         <HowToVote DisplayOrder="1">
            <Message Lang="en-US">Please see instructions online</Message>
         </HowToVote>
         <Rotation>no</Rotation>
         <RotationMethod/>
         <VotingMethod>FPP</VotingMethod>
         <ChoiceType>Checkbox</ChoiceType>
         <MaxVotes>1</MaxVotes>
         <MinVotes>1</MinVotes>
         <MaxWriteIn>1</MaxWriteIn>
         <Blocked>"no"</Blocked>
         <BallotChoices Contested="yes" Partisan="{./partisan}">
            <xsl:variable name="ballotID" select="./ballot_id"/>
            <xsl:apply-templates select="$VIPfeed//vip_object/ballot[@id=$ballotID]" mode="EML505"/>
         </BallotChoices>
         <Messages DisplayOrder="1">
            <Message Lang="en-US"/>
         </Messages>
         <ElectoralDistrictID><xsl:value-of select="./electoral_district_id"/></ElectoralDistrictID>
         <PrimaryParty><xsl:value-of select="./primary_party"/></PrimaryParty>
         <ContestOffice><xsl:value-of select="./office"/></ContestOffice>
         <BallotID><xsl:value-of select="./ballot_id"/></BallotID>
         <BallotPlacement><xsl:value-of select="./ballot_placement"/></BallotPlacement>
      </ContestType>
     </Contest>
   </xsl:template>
   
   <xsl:template match="ballot" mode="EML505">
      <xsl:apply-templates select="candidate_id" mode="EML505"/>
   </xsl:template>
   
   <xsl:template match="candidate_id" mode="EML505"><!-- this matches candidates on contests and lookups up the candidate details -->
      <xsl:variable name="candidateLookup" select="."/>
      <xsl:variable name="candidateInfo" select="$VIPfeed//vip_object/candidate[@id=$candidateLookup]"/>
      <xsl:if test="count($candidateInfo/name) &gt; 1"><xsl:message>ERROR: duplicate candidate id <xsl:value-of select="$candidateLookup"/> for contest.</xsl:message></xsl:if>
      <xsl:if test="count($candidateInfo/name) &lt; 1"><xsl:message>ERROR: missing candidate id <xsl:value-of select="$candidateLookup"/> for contest.</xsl:message></xsl:if>
      <Candidate DisplayOrder="{position()}" Independent="{if($candidateInfo/candidate/party = 'Independent') then 'Yes' else 'No'}"  xsl:exclude-result-prefixes="xal xnl">
         <CandidateIdentifier IdNumber="{.}" DisplayOrder="1" ShortCode="{.}">
            <CandidateName><xsl:value-of select="$candidateInfo/name"/></CandidateName>
         </CandidateIdentifier>
      </Candidate>
   </xsl:template>
   
   <xsl:template match="candidate" mode="EML505"><!-- this matches candidate details -->
      <Candidate DisplayOrder="{position()}" Independent="{if(party='Independent') then 'yes' else 'no'}"  xsl:exclude-result-prefixes="xal xnl">
         <CandidateIdentifier IdNumber="{@id}" DisplayOrder="1">
            <CandidateName><xsl:value-of select="name"/></CandidateName>
            <KnownAs/>
            <CandidateGender/>
            <ShortCode/>
         </CandidateIdentifier>
         <StatusDetails>
            <StatusItem setting="confirmed" date="{current-date()}"/>
         </StatusDetails>
         <DateOfBirth/>
         <Age/>
         <Gender/>
         <Contact DisplayOrder="{position()}" xsl:exclude-result-prefixes="xal xnl">
           <MailingAddress Type="Residence" ValidFrom="{current-date()}" LanguageCode="en-US" xsl:exclude-result-prefixes="xal xnl">
           <xal:PostalAddress xsl:exclude-result-prefixes="xal xnl">
            <xal:Thoroughfare number="{tokenize(filed_mailing_address/line1,' ')[1]}"><xsl:value-of select="filed_mailing_address/line1"/></xal:Thoroughfare>
            <xal:Locality type="PostTown"><xsl:value-of select="filed_mailing_address/city"/></xal:Locality> 
            <xal:PostCode type="ZIP"><xsl:value-of select="filed_mailing_address/zip"/></xal:PostCode>
            <xal:AdministrativeArea type="State"><xsl:value-of select="filed_mailing_address/state"/></xal:AdministrativeArea> 
            <xal:Country code="USA">United States</xal:Country> 
            </xal:PostalAddress>
          </MailingAddress>
            <Email><xsl:value-of select="email"/></Email> 
            <Telephone/>
            <Fax/>           
            <PreferredContact/> 
         </Contact>
         <Party>
            <AffiliationIdentifier>
               <RegisteredName><xsl:value-of select="party"/></RegisteredName>
            </AffiliationIdentifier>
            <Type>Party</Type>
            <Description/>
            <Logo/>
         </Party>
      </Candidate>
   </xsl:template>
   
   <xsl:template match="polling_location" mode="EML505"><!-- This reports the ballot summary details only -->
     <PollingLocation IdNumber="{@id}" DisplayOrder="{position()}"  xsl:exclude-result-prefixes="xal xnl">
      <PhysicalLocation IdNumber="{@id}">
         <PlaceName><xsl:value-of select="name"/></PlaceName>
         <Notes/>
         <Address type="{if(contains(upper-case(name),'SCHOOL')) then 'EducationalInstitution' else 'Facility'}">
            <xal:PostalAddress>
               <xal:Thoroughfare number="{tokenize(address/line1,' ')[1]}"><xsl:value-of select="address/line1"/></xal:Thoroughfare>
               <xal:Locality type="PostTown"><xsl:value-of select="address/city"/></xal:Locality>
               <xal:AdministrativeArea type="State"><xsl:value-of select="address/state"/></xal:AdministrativeArea>
               <xal:PostCode type="ZIP"><xsl:value-of select="address/zip"/></xal:PostCode>
               <xal:Country code="US">United States</xal:Country>
            </xal:PostalAddress>
            <xal:GeoRSS/>
         </Address>
         <PollingStation/>
         <Map/> 
      </PhysicalLocation>
      <TimeAvailable pollingEventType="Absentee_Ballot">
         <Start><xsl:value-of select="normalize-space(substring-before(polling_hours,'-'))"/></Start>
         <End><xsl:value-of select="normalize-space(substring-after(polling_hours,'-'))"/></End>
      </TimeAvailable>
      <ContestAssociation/>
      <ResultsReported/>
      <PollAssociation/>
   </PollingLocation>
   </xsl:template>
   
   <xsl:template match="street_segment" mode="EML505"><!-- This reports the ballot summary details only -->
     <LocalityBoundary PrecinctID="{precinct_id}" SplitID="{precinct_split_id}"  xsl:exclude-result-prefixes="xal xnl">
      <BoundaryMarker id="{@id}">
         <StartHouseNumber><xsl:value-of select="start_house_number"/></StartHouseNumber>
         <EndHouseNumber><xsl:value-of select="end_house_number"/></EndHouseNumber>
         <OddEvenBoth><xsl:value-of select="odd_even_both"/></OddEvenBoth>
         <NonHouseAddress  xsl:exclude-result-prefixes="xal xnl">
            <xal:PostalAddress>
               <xal:Thoroughfare type="{non_house_address/street_suffix}" name="{non_house_address/street_name}"><xsl:value-of select="concat(non_house_address/street_name,' ',non_house_address/street_suffix)"/></xal:Thoroughfare>
               <xal:Locality type="PostTown"><xsl:value-of select="non_house_address/city"/></xal:Locality>
               <xal:AdministrativeArea type="State"><xsl:value-of select="non_house_address/state"/></xal:AdministrativeArea>
               <xal:PostCode type="ZIP"><xsl:value-of select="non_house_address/zip"/></xal:PostCode>
               <xal:Country code="US">United States</xal:Country>
            </xal:PostalAddress>
            <xal:GeoRSS/>
         </NonHouseAddress>
      </BoundaryMarker>
     </LocalityBoundary>
   </xsl:template>
   
   <xsl:template match="ballot" mode="EML505a"><!-- This reports the ballot summary details only -->
      <Ballot id="{./@id}"  xsl:exclude-result-prefixes="xal xnl">
        <xsl:apply-templates select="candidate_id" mode="EML505a"/>
         <BallotMeasureID/>
      </Ballot>
   </xsl:template>
   
   <xsl:template match="candidate_id" mode="EML505a"><!-- this matches candidates on ballots and lookups up the candidate details -->
      <xsl:variable name="candidateLookup" select="."/>
      <xsl:variable name="candidateInfo" select="$VIPfeed//vip_object/candidate[@id=$candidateLookup]"/>
      <xsl:if test="count($candidateInfo/name) &gt; 1"><xsl:message>ERROR: duplicate candidate id <xsl:value-of select="$candidateLookup"/> for ballot.</xsl:message></xsl:if>
      <xsl:if test="count($candidateInfo/name) &lt; 1"><xsl:message>ERROR: missing candidate id <xsl:value-of select="$candidateLookup"/> for ballot.</xsl:message></xsl:if>
      <CandidateID IdNumber="{.}" DisplayOrder="1" ShortCode="{.}"  xsl:exclude-result-prefixes="xal xnl">
         <CandidateName><xsl:value-of select="$candidateInfo/name"/></CandidateName>
      </CandidateID>
   </xsl:template>
    
</xsl:stylesheet>
